#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ppm.h>

typedef double Func(double x);

double identity(double x) { return x; }
double square(double x) { return x*x; }

double calc_avg(char filename[], Func f, int channel)
{
    double avg=0.0;
    int cols,rows,format;
    pixval maxval;
    int yi,xi;
    FILE* file = fopen(filename,"r");
    if(!file) {
        fprintf(stderr,"Could not open %s for reading.\n",filename);
        exit(-1);
    }
    ppm_readppminit(file,&cols,&rows,&maxval,&format);
    pixel* pixelrow=ppm_allocrow(cols);
    for(yi=0;yi<rows;yi++) { 
        ppm_readppmrow(file,pixelrow,cols,maxval,format);
        for(xi=0;xi<cols;xi++) {
            pixel p=pixelrow[xi];
            double component = channel==0?p.r:channel==1?p.g:p.b;
            avg += f(component);
        }
    }
    fclose(file);
    avg/=cols*rows;
    return avg;
}

int main(int argc, char *argv[])
{
    int c; // channel
    double mean=0.0;
    char* filename;
    ppm_init(&argc,argv);
    if(argc!=1+1) {
        fprintf(stderr,"Usage: ppmsdev filename.ppm\n");
        exit(-1);
    }
    filename=argv[1];
    mean = 0.2989*calc_avg(filename,identity,0)
         + 0.5866*calc_avg(filename,identity,1)
         + 0.1145*calc_avg(filename,identity,2);
    printf("%f\n",mean);
    return 0;
}

