//
// ppmsdev: Compute the standard deviation of a ppm image.
//
// Written by Issac Trotts, April 18, 2006.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ppm.h>

typedef double Func(double x);

double identity(double x) { return x; }
double square(double x) { return x*x; }

double compute_empirical_expectation(char filename[], Func f, int channel)
{
    double avg=0.0;
    int cols,rows,format;
    pixval maxval;
    int yi,xi;
    FILE* file = fopen(filename,"r");
    if(!file) {
        fprintf(stderr,"Could not open %s for reading.\n",filename);
        exit(-1);
    }
    ppm_readppminit(file,&cols,&rows,&maxval,&format);
    pixel* pixelrow=ppm_allocrow(cols);
    for(yi=0;yi<rows;yi++) { 
        ppm_readppmrow(file,pixelrow,cols,maxval,format);
        for(xi=0;xi<cols;xi++) {
            pixel p=pixelrow[xi];
            double component = channel==0?p.r:channel==1?p.g:p.b;
            avg += f(component);
        }
    }
    fclose(file);
    avg/=cols*rows;
    return avg;
}

int main(int argc, char *argv[])
{
    int c; // channel
    double means[3], squares[3], vars[3], var, sdev;
    char* filename;
    ppm_init(&argc,argv);
    if(argc!=1+1) {
        fprintf(stderr,"Usage: ppmsdev filename.ppm\n");
        exit(-1);
    }
    filename=argv[1];
    for(c=0;c<3;c++) {
        means[c]=compute_empirical_expectation(filename,identity,c);
        squares[c]=compute_empirical_expectation(filename,square,c);
        vars[c]=squares[c]-square(means[c]);
    }
    var=(vars[0]+vars[1]+vars[2])/3;
    sdev=sqrt(var);
    printf("%f\n",sdev);
    return 0;
}

