char* usage=
"Usage: ppm-gabor-rms half_width_pix sdev_pix wavelength_pix  image.ppm\n"
"For example,\n"
"ppm-gabor-rms 10 3 3 image.ppm\n"
"\n"
"The program will print out the root-mean-square response of a Gabor\n"
"filter with given half-width in pixels, standard deviation in pixels,\n"
"and wavelength in pixels, over the given image.\n";

#include <ppm.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#define MAX_FILT 256

double square(double x) { return x*x; }

int main(int argc, char *argv[])
{
    int hwpix,nxfilt,nyfilt,nx,ny;
    float sdev,wavelen;
    float filt[MAX_FILT][MAX_FILT];
    pixval maxval;
    pixel** pixels;
    char* filename;

    // Parse command line args
    {
        if(argc!=1+4) {
            fprintf(stderr,usage);
            exit(-1);
        }
        hwpix = strtol(argv[1],NULL,10);
        sdev = (float)strtod(argv[2],NULL);
        wavelen = (float)strtod(argv[3],NULL);
        filename = argv[4];
        nxfilt = nyfilt = 2*hwpix+1;
        if(nxfilt>MAX_FILT) {
            fprintf(stderr,"Filter too large: %i > %i\n",nxfilt,MAX_FILT);
            exit(-1);
        }
    }

    // Construct the filter
    {
        int xi,yi;
        double x,y;
        for(yi=0;yi<nyfilt;yi++) {
            y = yi-(nyfilt-1)/2;
            for(xi=0;xi<nxfilt;xi++) {
                double val,r;
                x = xi-(nxfilt-1)/2;
                r = sqrt(x*x+y*y);
                val=exp(-0.5*square(r/sdev))*
                    0.5*(1.0+cos((2.0*M_PI/wavelen)*r));
                filt[yi][xi]=val;
            }
        }
    }

    // Load the image
    {
        FILE* file = fopen(filename,"rb");
        if(!file) {
            fprintf(stderr,"Could not open %s for reading.\n",filename);
            exit(-1);
        }
        pixels = ppm_readppm(file,&nx,&ny,&maxval);
        fclose(file);
    }

    // Compute and output the rms filter response.
    {
        int xi,yi;
        double result=0.0;
        for(yi=0;yi<ny;yi++) {
            for(xi=0;xi<nx;xi++) {
                double s=0.0,d=0.0;
                int xif,yif,yi2,xi2;
                for(yif=0;yif<nyfilt;yif++) {
                    yi2=yi+(yif-(nyfilt-1)/2);
                    for(xif=0;xif<nxfilt;xif++) {
                        xi2=xi+(xif-(nxfilt-1)/2);
                        if(yi2>=0&&yi2<ny&&xi2>=0&&xi2<nx) {
                            s+=filt[yif][xif]*PPM_LUMIN(pixels[yi2][xi2]);
                            d+=filt[yif][xif];
                        }
                    }
                }
                s/=d;
                result+=s*s;
            }
        }
        result=sqrt(result/(nx*ny));
        printf("%f\n",result);
    }

    return 0;
}


