//
// ppmentropy: Compute the total entropy of a ppm image's red, green and blue
// histograms.
//
// Written by Issac Trotts, April 18, 2006.
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ppm.h>

void calc_histograms(char filename[], unsigned **hr, unsigned **hg, unsigned **hb,
        unsigned *nbins)
{
    int cols,rows,format;
    pixval maxval;
    int yi,xi;
    FILE* file = fopen(filename,"r");
    if(!file) {
        fprintf(stderr,"Could not open %s for reading.\n",filename);
        exit(-1);
    }
    ppm_readppminit(file,&cols,&rows,&maxval,&format);
    *nbins=maxval+1;
    *hr=calloc((maxval+1),sizeof(unsigned));
    *hg=calloc((maxval+1),sizeof(unsigned));
    *hb=calloc((maxval+1),sizeof(unsigned));
    pixel* pixelrow=ppm_allocrow(cols);
    for(yi=0;yi<rows;yi++) { 
        ppm_readppmrow(file,pixelrow,cols,maxval,format);
        for(xi=0;xi<cols;xi++) {
            pixel p=pixelrow[xi];
            (*hr)[p.r]++;
            (*hg)[p.g]++;
            (*hb)[p.b]++;
        }
    }
    fclose(file);
}

// The result is in bits per pixel
double calc_entropy(unsigned h[],int nbins)
{
    int i;
    double s=0.0;
    for(i=0;i<nbins;i++) { s+=h[i]; }

    double *f=malloc(nbins*sizeof(double)); // frequencies
    for(i=0;i<nbins;i++) { f[i]=h[i]/s; }

    double ent=0.0;
    for(i=0;i<nbins;i++) { ent-=(f[i]==0.0)?0.0:f[i]*log(f[i])/log(2); }

    free(f);

    return ent;
}

int main(int argc, char *argv[])
{
    int c; // channel
    unsigned *hr,*hg,*hb, nbins;
    double ent=0.0;
    char* filename;
    ppm_init(&argc,argv);
    if(argc!=1+1) {
        fprintf(stderr,
"Usage: ppmentropy filename.ppm\n"
"\n"
"The output is the number of bits per pixel in a Shannon code based on the \n"
"marginal histograms of red, green, and blue.\n"
"\n");
        exit(-1);
    }
    filename=argv[1];
    calc_histograms(filename,&hr,&hg,&hb,&nbins);
    ent = calc_entropy(hr,nbins)
        + calc_entropy(hg,nbins)
        + calc_entropy(hb,nbins);
    free(hr); free(hg); free(hb);
    printf("%f\n",ent);
    return 0;
}

